class NoSuchIndexException(Exception):
    def __init__(self, key):
        self.key = key
        self.message = "No value with key "+str(key)+"exists."


class TimeoutException(Exception):
    def __init__(self):
        self.message = "Database timeout. "+ \
                       "If problem persists, contact admin."


class CassandraTypeException(Exception):
    def __init__(self, attempted_type, attempted_prop):
        self.type = attempted_type
        self.prop = attempted_prop
        self.message = "Type mismatch; Cassandra does not anticipate type "+ \
                       self.type+" for property "+self.prop


class NoSuchColumnFamilyException(Exception):
    def __init__(self, cf):
        self.cf = cf
        self.message = "No such column family "+self.cf+" exists. Try building it."


class NoSuchColumnException(Exception):
    def __init__(self, column):
        self.column = column
        self.message = "Column "+self.column+" does not exists."


from uuid import uuid4
from base import Base
import db_utils as db

CASSANDRA_KEYSPACE = 'gradfly'
CASSANDRA_IP = ['192.241.181.163', '107.170.88.98']


class LookUp(Base):
    __tablename__ = "lookuptable"

    defaults = {
        "id": "",
        "model": "",
        "model_id": uuid4()
    }

    def __init__(self, *args, **kwargs):
        self.update_data(**self.defaults)
        Base.__init__(self, *args, **kwargs)

    @staticmethod
    def _build(__tablename__):
        build = "create table "+CASSANDRA_KEYSPACE+"."+__tablename__
        build += " ( id varchar PRIMARY KEY, model varchar, model_id uuid ) with caching = \'all\'"
        db.c.session.execute(build)
        db.c.session.execute("create index model on "
                             +CASSANDRA_KEYSPACE+"."+__tablename__+" (model)")

    @classmethod
    def build(cls):
        cls._build(cls.__tablename__)

    @classmethod
    def drop(cls):
        cls._drop(cls.__tablename__)

    @classmethod
    def rebuild(cls):
        try:
            cls.drop()
        except Exception:
            pass
        cls.build()



from uuid import uuid4
from maelstrom.data import Data
from test import CassandraTestCase


class DBUnitTests(CassandraTestCase):
    """
    The Data class is the generic protoype for the database models. As such,
    correct testing database functionality on it will test database
    transactions, while other model-specific tests should be written
    in a separate testing class.
    """
    test_setup = False

    def setUp(self):
        self.ids_used = []
        CassandraTestCase.setUp(self, ['192.241.181.163', '107.170.88.98'], 'gradfly')
        #c.start(['192.241.181.163', '107.170.88.98'], 'gradfly')
        '''
        try:
            c.start(['192.241.181.163', '107.170.88.98'], 'gradfly')
        except Exception:
            from time import sleep
            print 'failed'
            sleep(50000)
        finally:
            c.start(['192.241.181.163', '107.170.88.98'], 'gradfly')
        '''
    def test_get_and_put_by_id(self):
        init_id = uuid4()
        self.ids_used.append(init_id)
        data = Data(id=init_id, contents="john ")
        data.commit()
        test_data = Data.get_by_id(init_id)
        #for i,j in zip(user_1.__dict__.values(), test_user_1.__dict__.values()):
        #	print i,j, i == j
        self.assertEqual(data, test_data)

    def test_batch_and_get(self):
        ids = [uuid4(), uuid4(), uuid4()]
        self.ids_used += list(ids)
        names = ['bob', 'joe', 'sara']
        datum = [Data(id=i, contents=name) for i, name in zip(ids, names)]
        datum = {a.id: a for a in datum}

        Data.multi_update_data(datum)
        datum_test = Data.multi_get_by_id(ids).values()

        #no order is guaranteed by Cassandra so we sort it
        datum_test.sort(key=(lambda a: a.id))
        datum_true = sorted(datum.values(), key=lambda a: a.id)

        if len(ids) != len(datum):
            self.assertTrue(False)
        print datum_true
        print datum_test
        is_equal = True
        for a, a_test in zip(datum_true, datum_test):
            is_equal = is_equal and (a == a_test)
        self.assertTrue(is_equal)

    def test_batch_vs_iter_single(self):
        ids = [uuid4(), uuid4(), uuid4()]
        self.ids_used += list(ids)
        names = ['bob', 'joe', 'sara']
        datum = [Data(id=i, contents=name) for i, name in zip(ids, names)]
        datum = {a.id: a for a in datum}

        Data.multi_update_data(datum)
        datum_batch = Data.multi_get_by_id(ids).values()
        datum_single = []

        for i in ids:
            datum_single.append(Data.get_by_id(i))
        datum_batch.sort(key=(lambda a: a.id))
        datum_single.sort(key=(lambda a: a.id))
        print datum_single
        print datum_batch
        self.assertTrue(datum_batch == datum_single)

    def test_delete(self):
        init_id = uuid4()
        self.ids_used.append(init_id)
        data = Data(id=init_id, contents="testtesttest")
        data.commit()
        data.delete(init_id)
        if not Data.get_by_id(init_id):
            self.assertTrue(True)
        else:
            self.assertTrue(False)

    def tearDown(self):
        Data.multi_delete(self.ids_used)
        CassandraTestCase.tearDown(self)
        #c.stop()




from base import Base
import db_utils as db
from cassandra.query import SimpleStatement
from cassandra import ConsistencyLevel
import cassandra
from uuid import uuid4
from itertools import izip

CASSANDRA_KEYSPACE = 'gradfly'
CASSANDRA_IP = ['192.241.181.163', '107.170.88.98']

class Search(Base):
    __tablename__ = "search"

    defaults = {
        "id" : "",
        "model_type" : "",
        "model_ids" : set([uuid4()])
    }


    def __init__(self, *args, **kwargs):
        self.model_ids = set()
        self.update_data(**self.defaults)
        Base.__init__(self, *args, **kwargs)

    @staticmethod
    def _build(__tablename__):
        build = "create table "+CASSANDRA_KEYSPACE+"."+__tablename__
        build += " ( id varchar PRIMARY KEY, model_type varchar, model_ids set<uuid> ) with caching = \'all\'"
        db.c.session.execute(build)
        db.c.session.execute("create index model_type on "
                +CASSANDRA_KEYSPACE+"."+__tablename__+" (model_type)")

    @classmethod
    def build(cls):
        cls._build(cls.__tablename__)

    @classmethod
    def drop(cls):
        cls._drop(cls.__tablename__)

    @classmethod
    def rebuild(cls):
        try:
            cls.drop()
        except Exception:
            pass
        cls.build()

    @staticmethod
    def _build_insert_query(__tablename__, id, **kwargs):
        query = "update "+CASSANDRA_KEYSPACE+"."+__tablename__
        query += " set model_type=\'"+kwargs['model_type']+"\'"
        query += ", model_ids = model_ids + { "
        for model_id in kwargs['model_ids']:
            query += str(model_id)+", " 
        query = query[:-2] + "} where id=\'"+id+"\'"
        return query

    @staticmethod
    def _set_row(__tablename__, id, **kwargs):
        query = _build_insert_query(__tablename__, id, **kwargs)
        return db.c.session.execute(
            SimpleStatement(query, consistency_level=ConsistencyLevel.QUORUM))

    @staticmethod
    def _set_rows(__tablename__, ids, rows):

        list_of_kwargs = [r.__dict__ for r in rows]
        #typechecking   
        batch = "begin batch "
        for uuid, kwargs in izip(ids, list_of_kwargs):
            batch += Search._build_insert_query(__tablename__, **kwargs)+" "
        batch += "apply batch"
        try:
            db.c.session.execute(
                SimpleStatement(batch, consistency_level=ConsistencyLevel.QUORUM)
            )
        except cassandra.InvalidRequest:
            print "ColumnFamilyError: no such column family exists."

    @classmethod
    def _multi_commit(cls, list_of_rows):
        ids = []
        kwargs = []
        for row in list_of_rows:
            ids.append(row.id)
            kwargs.append(row.__dict__)
        cls._set_rows(cls.__tablename__, ids, kwargs)

    def _commit(self):
        self._set_row(self.__tablename__, **self.__dict__)



from uuid import uuid4
from maelstrom.base import Base
from maelstrom.db_utils import get_time


class Account(Base):
    defaults = {
        'id': uuid4(),
        'follower_count': 0,
        'following_count': 0,
        'stars_count': 0,
        'school_id': uuid4(),
        'login_count': 0,
        'permission': 0,
        'state': 0,
        'email_authorized': False,
        'account_suspended': False,
        'email': '',
        'password_hash': '',
        'password_salt': '',
        'current_login_ip': '',
        'last_login_ip': '',
        'school': '',
        'username': '',
        'name': '',
        'major': '',
        'grade': '',
        'city_name': '',
        'state_name': '',
        'bio_text': '',
        'avatar_url': '',
        'following_url': '',
        'follower_url': '',
        'last_login_at': get_time(),
        'updated_at': get_time(),
        'created_at': get_time(),
        'birthday': get_time(),
        'trophies': [uuid4()],
        'organizations': [uuid4()],
        'comments': [uuid4()],
        'likes': [uuid4()],
        'tags': [uuid4()],
        'subscribed': [uuid4()],
        'subscribers': [uuid4()],
        'following_projects': [uuid4()],
        'projects': [uuid4()]
    }

    lookups = ['username', 'email']
    search_terms = ['username', 'name', 'email']

    __tablename__ = 'users'

    def __init__(self, *args, **kwargs):
        self.update_data(**self.defaults)
        Base.__init__(self, *args, **kwargs)

    def is_authenticated(self):
        return self.email_authorized

    def is_active(self):
        return not self.account_suspended

    def is_anonymous(self):
        return False

    def get_id(self):
        return self.id

    @property
    def trophy_count(self):
        return len(self.trophies)

    @property
    def following_count(self):
        return len(self.subscribed)

    @property
    def followers_count(self):
        return len(self.subscribers)

    def get_id(self):
        return self.id

    # TODO
    def is_following_user(self, account_id):
        return account_id in self.subscribed

    # TODO
    def is_subscribed_project(self, project_id):
        return project_id in self.following_projects

    # TODO
    def Projects(self):
        from gf.models.project import Project

        projects = Project.multi_get_by_id(self.projects)
        project_list = []
        for project in projects:
            project_list.append(project)
        return project_list



from datetime import datetime
import db_utils as db


class Row(object):

    _get_row_by_id = staticmethod(db.get_row_by_id)
    _get_row_spec_prop = staticmethod(db.get_row_spec_prop)
    _get_rows_by_id = staticmethod(db.get_rows_by_id)
    _set_row = staticmethod(db.insert_row_data)
    _set_rows = staticmethod(db.set_rows_data)
    _build = staticmethod(db.create_column_family)
    _drop = staticmethod(db.drop_column_family)
    _delete = staticmethod(db.delete_row)
    _multi_delete = staticmethod(db.delete_rows)
    
    id = ''
    __tablename__ = ''
    defaults = {}

    def __init__(self, unique_id, *args, **kwargs):
        for k in kwargs:
            if k == 'id':continue
            if type(self.defaults[k]) != type(kwargs[k]):
                if type(kwargs[k]) == datetime:
                    kwargs[k] = db.to_epoch(kwargs[k])
                    continue
                kwargs[k] = type(self.defaults[k])(kwargs[k])
        self.__dict__.update(kwargs)
        self.id = unique_id

    def __getattr__(self, name):
        print name
        object.__getattr__(self, name)

    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)

    def __repr__(self):
        s = '<'+str(self.__class__.__name__)+' '
        for k in sorted(self.__dict__.iterkeys()):
            if type(self.__dict__[k]) == datetime:
                continue
            s += str(k)+': '+str(self.__dict__[k])+', '
        return s[:-2]+'>'

    def __eq__(self, other):
        return self.__dict__ == other.__dict__
    '''
    @classmethod
    def _query(cls, row_type, *rules, **kwargs):
        return Query(row_type, *rules, **kwargs)
    '''

    @classmethod
    def _multi_commit(cls, list_of_rows):
        ids = []
        kwargs = []
        for row in list_of_rows:
            ids.append(row.id)
            kwargs.append(row.__dict__)
        cls._set_rows(cls.__tablename__, ids, kwargs)

    @classmethod
    def build(cls):
        cls._build(cls.__tablename__, **cls.defaults)

    @classmethod
    def drop(cls):
        cls._drop(cls.__tablename__)

    @classmethod
    def rebuild(cls):
        try:
            cls.drop()
        except Exception as e:
            pass
        cls.build()

    def _update(self, **kwargs):
        self.__dict__.update(kwargs)

    def _commit(self):
        self._set_row(self.__tablename__, self.id, **self.__dict__)






from row import Row
from exceptions import NoSuchIndexException


class Base(Row):
    lookups = []
    search_terms = []

    def __init__(self, *args, **kwargs):
        if 'id' in kwargs.keys():
            unique_id = kwargs['id']
        else:
            unique_id = self.id
        try:
            Row.__init__(self, unique_id, *args, **kwargs)
        except KeyError as k:
            print "KeyError: Column", k, "does not exist."
        finally:
            return

    def __setattr__(self, name, value):
        super(Base, self).__setattr__(name, value)

    def __getattr__(self, name):
        super(Base, self).__getattr__(name)

    @classmethod
    def get_by_lookup(cls, string):
        from gf.lib.db.lookup import LookUp
        base = None
        try:
            base = LookUp.get_by_id(string)            
        except NoSuchIndexException:
            return None

        unique_id = base.model_id
        return cls.get_by_id(unique_id)

    @classmethod
    def get_by_id(cls, req_id):
        data = None
        try:
            data = cls._get_row_by_id(cls.__tablename__, req_id)
        except NoSuchIndexException:
            return None
        return cls(**data)

    @classmethod
    def get_specific_props(cls, req_id, columns):
        data = cls._get_row_spec_prop(cls.__tablename__, req_id,
                                      columns+['id'])
        return cls(**data)

    @classmethod
    def multi_get_by_id(cls, req_ids):
        rows = cls._get_rows_by_id(cls.__tablename__, req_ids)
        objs = {}
        for row in rows:
            objs[row['id']] = cls(**row)
        return objs

    @classmethod
    def multi_update_data(cls, dict_of_sessions):
        uuids = dict_of_sessions.keys()
        list_of_sessions = dict_of_sessions.values()
        cls._set_rows(cls.__tablename__, uuids, list_of_sessions)

    @classmethod
    def delete(cls, id):
        cls._delete(cls.__tablename__, id)
        # TODO
        """
        if cls.lookups:
            from gf.lib.db.lookup import LookUp
            cls._multi_delete(LookUp.__tablename__, cls.lookups)
        if cls.search_terms:
            from gf.lib.db.search import Search
            cls._multi_delete(Search.__tablename__, cls.search_terms)
        """

    @classmethod
    def multi_delete(cls, ids):
        for id in ids:
            cls.delete(id)

    def update_data(self, **kwargs):
        self._update(**kwargs)

    def commit(self):
        from .lookup import LookUp
        from .search import Search

        to_lookups = {}
        to_search = {}
        if self.lookups:
            for lookup in self.lookups:
                lookup_id = self.__dict__[lookup]
                if lookup_id != "":
                    to_lookups[lookup_id] = LookUp(id=lookup_id,
                                                   model=self.__tablename__,
                                                   model_id=self.id)
            LookUp.multi_update_data(to_lookups)

        if self.search_terms:
            for term in self.search_terms:
                search_string = self.__dict__[term]
                if search_string != "":
                    substring = ""
                    for char in search_string:
                        substring += char
                        to_search[substring] = Search(id=substring,
                                                      model_type=self.__tablename__,
                                                      model_ids=set([self.id]))
                        #print to_search
            print to_search
            Search.multi_update_data(to_search)

        self._commit()





from multiprocessing import cpu_count
from cassandra.cluster import Cluster, NoHostAvailable
from cassandra.decoder import dict_factory


class CassandraConnection(object):

    ip = None
    kp = None

    def __init__(self, cass_ip, cass_kp):
        self.ip = cass_ip
        self.kp = cass_kp
        try:
            #self.cluster = Cluster(contact_points=self.ip, control_connection_timeout=10000.0)
            self.cluster = Cluster(contact_points=self.ip)
            #self.session = self.cluster.connect(self.kp)
            self.session = TransactionManager(self)
            self.session.row_factory = dict_factory
        except NoHostAvailable:
            print "TimeoutError: Possibly a connection issue."

    def destroy_cluster(self):
        ip = None
        kp = None
        self.cluster.shutdown()
        self.session.shutdown()


class TransactionManager:
    sessions = [] 
    
    def __init__(self, connector):
        self.connector = connector
        for i in range(cpu_count() - 1):
            session = self.connector.cluster.connect(self.connector.kp)
            session.row_factory = dict_factory
            self.sessions.append(session)
        print self.sessions
    

    def execute(self, *args, **kwargs):
        current_session = self.sessions.pop(0)
        result = current_session.execute_async(*args, **kwargs)
        self.sessions.append(current_session)
        return result

    def shutdown(self):
        for session in self.sessions:
            session.shutdown()



from base import Base
from uuid import uuid4


class Data(Base):
    __tablename__ = "data"

    defaults = {
        "id": uuid4(),
        "contents": ""
    }

    def __init__(self, *args, **kwargs):
        self.update_data(**self.defaults)
        Base.__init__(self, *args, **kwargs)


import unittest
import maelstrom as c


class CassandraTestCase(unittest.TestCase):

    def setUp(self, cass_ip, cass_kp):
        if not self.test_setup:
            c.start(cass_ip, cass_kp)
            self.__class__.test_setup = True

    def tearDown(self):
        pass


from uuid import uuid4
from maelstrom.search import Search
from account import Account
from test import CassandraTestCase


class SearchUnitTests(CassandraTestCase):

    test_setup = False

    def setUp(self):
        CassandraTestCase.setUp(self, ['192.241.181.163', '107.170.88.98'], 'gradfly')
        #c.start(['192.241.181.163', '107.170.88.98'], 'gradfly')
        '''
        try:
            c.start(['192.241.181.163', '107.170.88.98'], 'gradfly')
        except Exception:
            from time import sleep
            print 'failed :('
            sleep(50000)
        finally:
            c.start(['192.241.181.163', '107.170.88.98'], 'gradfly')
        '''
        self.init_id = uuid4()
        self.account = Account(id=self.init_id, name='Matt Morse',
                               username='matt',
                               email='matt@gradf.ly',
                               login_count=80,
                               email_authorized=True,
                               grade='14',
                               city_name='Buffalo',
                               bio_text='THE MOST BADASS STEM STUDENT IN TOWN',
                               tags=[uuid4(), uuid4(), uuid4(), uuid4(),
                                     uuid4()],
                               subscribed=[uuid4(), uuid4()],
                               subscribers=[uuid4(), uuid4()],
                               projects=[uuid4()],
                               following_projects=[uuid4(), uuid4()],
                               trophies=[uuid4(), uuid4(), uuid4()])
        self.account.commit()

    def test_search_username(self):
        search = Search.get_by_id("matt")
        self.assertTrue(self.account.id in search.model_ids)

    def test_search_email(self):
        search = Search.get_by_id("matt@gradf.ly")
        self.assertTrue(self.account.id in search.model_ids)

    def test_search_username_substr(self):
        search = Search.get_by_id("ma")
        print self.account.id
        print search.model_ids
        self.assertTrue(self.account.id in search.model_ids)

    def test_search_email_substr(self):
        search = Search.get_by_id("matt@g")
        print self.account.id
        print search.model_ids
        self.assertTrue(self.account.id in search.model_ids)

    def test_search_refined(self):
        search_general = Search.get_by_id("m")
        search_username = Search.get_by_id("matt")
        search_email = Search.get_by_id("matt@gradf.ly")
        username_is_subset = search_username.model_ids.issubset(
            search_general.model_ids)
        email_is_subset = search_email.model_ids.issubset(
            search_general.model_ids)
        self.assertTrue(username_is_subset and email_is_subset)

    def tearDown(self):
        CassandraTestCase.tearDown(self)
        #c.stop()



from setuptools import setup

setup(name='maelstrom-py',
      version='0.1.2a',
      author='Matt Morse, Joe Peacock',
      author_email='mmorse1217@gmail.com, joeapeacock@gmail.com',
      packages=['maelstrom', 'maelstrom.tests'],
      url='https://github.com/gradfly/maelstrom',
      license='LICENSE',
      description='A powerful, model based Cassandra wrapper for Python.',
      long_description=open('README.rst').read(),
      install_requires=[
          "cassandra-driver == 2.0.2",
          "nose >= 1.3.1",
          "rednose"
      ],
      test_suite="nose.collector")



"""

Associated database objects and functions that are critical to
database interactions. Functions are defined here are used in
the generic data objects to improve the modularity of the sytem
and allow for easier future development. Classes defined here
are meant to be used only by the module's functions and should
not be instantiated client-side.

"""

from datetime import date, datetime
from uuid import UUID
from collections import OrderedDict
from decimal import Decimal
from cassandra import InvalidRequest, ConsistencyLevel
import cassandra
from cassandra.query import SimpleStatement
from itertools import izip
from time import time
from .exceptions import NoSuchIndexException, NoSuchColumnFamilyException
from .cass_conn import CassandraConnection


c = None
cass_keyspace = None


def connect(cass_ip, cass_kp):
    global c, cass_keyspace
    cass_keyspace = cass_kp
    c = CassandraConnection(cass_ip, cass_kp)


def close():
    c.destroy_cluster()


def get_type_map():
    return {None: 'Null',
            bool: 'boolean',
            float: 'double',
            int: 'bigint',
            long: 'bigint',
            Decimal: 'decimal',
            str: 'varchar',
            unicode: 'varchar',
            buffer: 'blob',
            bytearray: 'blob',
            date: 'timestamp',
            datetime: 'timestamp',
            list: 'list',
            tuple: 'list',
            set: 'set',
            dict: 'map',
            OrderedDict: 'map',
            UUID: 'uuid'}


def is_collection(v):
    return type(v) == list or type(v) == dict or type(v) == set


def get_time():
    """
    cassie-driver autoserializes timestamp columns into Python datetime
    objects datetime does not trivially translate into seconds from epoch,
    which is needed to guarantee uniqueness of objects. Subject to change at a
    later time.
    """
    return int(time() * 100)


def to_epoch(d_time):
    return int((d_time-datetime(1970, 1, 1)).total_seconds() * 1000)


def create_column_family(cf_name, **kwargs):
    type_map = get_type_map()
    build = "create table "+cass_keyspace+"."+cf_name+" ( "
    build += "id uuid PRIMARY KEY, "
    for k, v in kwargs.iteritems():
        if k == "id":
            continue
        if is_collection(v):
            if type(v) == dict:
                pass
                #typechecking
            else:
                #typechecking
                pass
            build += k+" "+type_map[type(v)]+'<'+type_map[type(v[0])]+">, "
        else:
            build += k+" "+type_map[type(v)]+", "
    build = build[:-2]+") with caching = \'all\'"
    c.session.execute(build)


def drop_column_family(cf_name):
    c.session.execute('drop table '+cass_keyspace+"."+cf_name)


def get_row_by_id(cf_name, row_id):
    query = SimpleStatement('select * from '+cf_name+' where id = %s',
                            consistency_level=cassandra.ConsistencyLevel.QUORUM)
    try:
        print 'select * from '+cf_name+' where id =', row_id
        return c.session.execute(query, [row_id]).result()[0]
    except InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)
    except IndexError:
        raise NoSuchIndexException(row_id)


def get_rows_by_id(cf_name, row_ids):
    query = "select * from "+cf_name+" where id in ("
    for rid in row_ids:
        query += str(rid)+","
    query = query[:-1]+")"
    try:
        return c.session.execute(
            SimpleStatement(query, consistency_level=ConsistencyLevel.QUORUM)).result()
    except InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)
    except IndexError:
        raise NoSuchIndexException(row_ids)


def build_insert_query(cf_name, unique_id, **kwargs):
    cols = kwargs.keys()
    vals = kwargs.values()
    query = "insert into "+cf_name+" ("
    for col in cols:
        query += str(col)+", "
    query = query[:-2]+") values ("
    for val in vals:
        if type(val) == list:
            query += ' ['
            for element in val:
                query += str(element)+', '
            query = query[:-2]+'], '
        elif type(val) == str or type(val) == datetime:
            query += '\''+str(val)+"', "
        elif type(val) == UUID:
            query += str(val)+", "

        else:
            query += str(val)+", "

    query = query[:-2]+")"
    return query


def insert_row_data(cf_name, unique_id, **kwargs):
    query = build_insert_query(cf_name, unique_id, **kwargs)
    try:
        return c.session.execute(
            SimpleStatement(query, consistency_level=ConsistencyLevel.QUORUM)).result()
    except InvalidRequest:
        print "ColumnFamilyError: no such column family exists."
    except InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)


def set_rows_data(cf_name, unique_ids, list_of_rows):
    list_of_kwargs = [r.__dict__ for r in list_of_rows]
    batch = "begin batch "
    for uuid, kwargs in izip(unique_ids, list_of_kwargs):
        batch += build_insert_query(cf_name, uuid, **kwargs)+" "
    batch += "apply batch"
    try:
        c.session.execute(
            SimpleStatement(batch, consistency_level=ConsistencyLevel.QUORUM)
        ).result()
    except InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)


def get_row_spec_prop(cf_name, unique_id, cols):
    query = "select "
    for col in cols:
        query += col+", "
    query = query[:-2]+" where id=%s", (unique_id)
    try:
        return c.session.execute(
            SimpleStatement(query,
                            consistency_level=cassandra.ConsistencyLevel.QUORUM))[0]
    except cassandra.InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)
    except IndexError:
        raise NoSuchIndexException(unique_id)


def delete_row(cf_name, unique_id):
    query = "delete from "+cf_name+" where id="+str(unique_id)
    try:
        c.session.execute(
            SimpleStatement(query,
                            consistency_level=cassandra.ConsistencyLevel.QUORUM)
        )
    except cassandra.InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)
    except IndexError:
        raise NoSuchIndexException(unique_id)


def delete_rows(cf_name, unique_ids):
    batch = "begin batch "
    for id in unique_ids:
        batch += "delete from "+cf_name+" where id="+str(id)+" "
    batch += "apply batch"
    try:
        c.session.execute(
            SimpleStatement(batch,
                            consistency_level=cassandra.ConsistencyLevel.QUORUM)
        )
    except cassandra.InvalidRequest:
        raise NoSuchColumnFamilyException(cf_name)
    except IndexError:
        raise NoSuchIndexException(unique_ids)

